import { tree } from 'd3';

export const nodeLinkTree = (
  selection,
  { root, width, height }
) => {
  tree().size([width - 20, height - 50])(root);

  const colorScale = d3
    .scaleOrdinal()
    .domain(
      root.descendants().map((d) => d.data.name)
    )
    .range([
      'black',
      '#fc8d62',
      '#617ec0',
      '#fc8d62b8',
      '#fc8d626e',
      '#fc8d626e',
      '#617ec0',
      '#617ec0b0',
      '#617ec073',
      '#617ec047',
      '#617ec029',
      '#617ec00f',
    ]);

  const line = selection
    .selectAll('line')
    .data(root.links())
    .join('line')
    .attr('x1', (d) => d.source.x)
    .attr('y1', (d) => d.source.y + 10)
    .attr('x2', (d) => d.target.x)
    .attr('y2', (d) => d.target.y + 10)
    .attr('stroke', 'black');

  const circle = selection
    .selectAll('circle')
    .data(root.descendants())
    .join('circle')
    .attr('cx', (d) => d.x)
    .attr('cy', (d) => d.y + 20)
    .attr('r', 10)
    .style('fill', (d) => colorScale(d.data.name))
    .on('mouseover', function (event, d) {
      // Highlight connected nodes and links
      const connected = new Set();
      root.links().forEach((l) => {
        if (l.source === d || l.target === d) {
          connected.add(l.source);
          connected.add(l.target);
        }
      });
      line
        .style('stroke', (l) =>
          l.source === d || l.target === d
            ? '#999'
            : 'black'
        )
        .style('stroke-width', (l) =>
          l.source === d || l.target === d ? 2 : 1
        );
      circle
        .style('fill', (n) =>
          connected.has(n)
            ? colorScale(n.data.name)
            : '#ddd'
        )
        .style('stroke', (n) =>
          connected.has(n) ? '#fff' : null
        );

      // Show tooltip
      const tooltip = selection
        .append('g')
        .attr('id', 'tooltip');
      const tooltipWidth = 200;
      const tooltipHeight = 20;
      const mouseX = event.clientX;
      const mouseY = event.clientY;
      const tooltipX =
        d.x + tooltipWidth > width
          ? d.x - tooltipWidth
          : d.x;
      const tooltipY =
        d.y + tooltipHeight < height
          ? d.y
          : height - tooltipHeight;
      tooltip
        .append('rect')
        .attr('x', tooltipX)
        .attr('y', tooltipY)
        .attr('width', tooltipWidth)
        .attr('height', tooltipHeight)
        .attr('rx', 10)
        .attr('ry', 10)
        .style('fill', '#333')
        .style('opacity', 1);
      tooltip
        .append('text')
        .attr('x', tooltipX + tooltipWidth / 2)
        .attr(
          'y',
          tooltipY + tooltipHeight / 2 + 5
        )
        .attr('text-anchor', 'middle')
        .style('font-family', 'sans-serif')
        .style('fill', '#fff')
        .text(d.data.name);
  });

  circle.on('mouseout', function (event, d) {
    selection.select('#tooltip').remove();
  });

  circle.on('mouseout', function (event, d) {
    selection.select('#tooltip').remove();
  });
};
