import {
  select,
  schemeCategory10,
  csvParse,
  min,
  max,
  extent,
  group,
  flatGroup,
  rollup,
  flatRollup,
  sum,
  bin,
  timeFormat,
} from 'd3';

import { pieChart } from './pie2';

export const viz3 = (
  container,
  { state, setState }
) => {
  const width = window.innerWidth / 2;
  const height = window.innerHeight / 2;

  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height);

  // state.data could be:
  // * undefined
  // * 'LOADING'
  // * An array of objects
 const { data, hoveredValue } = state;

  const setHoveredValue = (d) => {
    setState((state) => ({
      ...state,
      hoveredValue: d,
    }));
  };
  
  if (data && data !== 'LOADING') {
    svg.call(pieChart, {
      data,
      width,
      height
    });
  }

  if (data === undefined) {
    setState((state) => ({
      ...state,
      data: 'LOADING',
    }));

    fetch('data.csv')
      .then((response) => response.text())
      .then((csvString) => {
        const data = csvParse(csvString);
        var parseTime = d3.timeParse('%Y');
        for (const d of data) {
          d.Date = parseTime(
            d.Date.split('-')[0]
          );
        }
        const group = flatGroup(
          data,
          (d) => d.Department
        ).map(([key, value]) => ({
          department: key,
          count: value.length,
        }));
      
        const summary = group
          .slice()
          .sort((a, b) =>
            d3.descending(a.count, b.count)
          );
      console.log(summary)

        setState((state) => ({
          ...state,
          data: summary,
        }));
      });
  }
};
